import type { PredIndexed } from "./_types";
/**
 * Converts a list of objects into an object indexing the objects by the given key (casted to a string).
 * Use the strict version to maintain the given key's type, so long as it is a valid `PropertyKey`.
 *
 * @param array the array
 * @param fn the indexing function
 * @signature
 *    R.indexBy(array, fn)
 *    R.indexBy.strict(array, fn)
 * @example
 *    R.indexBy(['one', 'two', 'three'], x => x.length) // => {"3": 'two', "5": 'three'}
 *    R.indexBy.strict(['one', 'two', 'three'], x => x.length) // => {3: 'two', 5: 'three'}
 * @dataFirst
 * @indexed
 * @category Array
 * @strict
 */
export declare function indexBy<T>(array: ReadonlyArray<T>, fn: (item: T) => unknown): Record<string, T>;
/**
 * Converts a list of objects into an object indexing the objects by the given key.
 * (casted to a string). Use the strict version to maintain the given key's type, so
 * long as it is a valid `PropertyKey`.
 *
 * @param array the array
 * @param fn the indexing function
 * @signature
 *    R.indexBy(fn)(array)
 *    R.indexBy.strict(fn)(array)
 * @example
 *    R.pipe(
 *      ['one', 'two', 'three'],
 *      R.indexBy(x => x.length)
 *    ) // => {"3": 'two', "5": 'three'}
 *    R.pipe(
 *      ['one', 'two', 'three'],
 *      R.indexBy.strict(x => x.length)
 *    ) // => {3: 'two', 5: 'three'}
 * @dataLast
 * @indexed
 * @category Array
 * @strict
 */
export declare function indexBy<T>(fn: (item: T) => unknown): (array: ReadonlyArray<T>) => Record<string, T>;
declare function indexByStrict<K extends PropertyKey, T>(array: ReadonlyArray<T>, fn: (item: T) => K): Partial<Record<K, T>>;
declare function indexByStrict<K extends PropertyKey, T>(fn: (item: T) => K): (array: ReadonlyArray<T>) => Partial<Record<K, T>>;
export declare namespace indexBy {
    function indexed<T>(array: ReadonlyArray<T>, fn: PredIndexed<T, unknown>): Record<string, T>;
    function indexed<T>(fn: PredIndexed<T, unknown>): (array: ReadonlyArray<T>) => Record<string, T>;
    const strict: typeof indexByStrict;
}
export {};
//# sourceMappingURL=indexBy.d.ts.map